%Clohessy-Wiltshire equations solution

%state vector is [x, y, z, xdot, ydot, zdot]^T

clear all
clc
close all

%symbolic variables
om=sym('om','real');
t=sym('t','real');
A=sym('A','real');
X0=sym('X0','real');
x0=sym('x0','real');
y0=sym('y0','real');
z0=sym('z0','real');
xdot0=sym('xdot0','real');
ydot0=sym('ydot0','real');
zdot0=sym('zdot0','real');
X0=sym('X0','real');
X=sym('X','real');
phi=sym('phi','real');
phi_rr=sym('phi_rr','real');
phi_rv=sym('phi_rv','real');
X0 = [x0; y0; z0; xdot0; ydot0; zdot0];

%om = 0.01; %rad/sec (orbital angular velocity of chief satellite)
A = [0 0 0 1 0 0;
     0 0 0 0 1 0;
     0 0 0 0 0 1;
    3*om^2 0 0 0 2*om 0;
    0 0 0 -2*om 0 0;
    0 0 -om^2 0 0 0];

X = simplify(expm(A*t)*X0)


%computing symbolically the impulse to reach origin
phi = expm(A*t);
phi_rr = phi(1:3, 1:3);
phi_rv = phi(1:3, 4:6);
vel0new = -simplify(inv(phi_rv)*phi_rr*[x0; y0; z0])

%numerical example
om=0.001; %rad/s
T=2*pi/om;
x0=5; %km
y0=2; %km
z0=1; %km
xdot0=0.01; %km/s
ydot0=-2*om*x0; %km/s; -2*om*x0;
zdot0=0.02;
tf = (3/2)*T;

% %initial velocity to go to the origin
% xdot0 = -((om*(2*y0 - 2*y0*cos(om*tf) - 4*x0*sin(om*tf) + 3*om*tf*x0*cos(om*tf)))/(8*cos(om*tf) + 3*om*tf*sin(om*tf) - 8));
% ydot0 = -(-(om*(14*x0 - 14*x0*cos(om*tf) + y0*sin(om*tf) - 6*om*tf*x0*sin(om*tf)))/(8*cos(om*tf) + 3*om*tf*sin(om*tf) - 8));
% zdot0 = -(om*z0*cos(om*tf))/sin(om*tf);                                                                          

%analytical solution
t = linspace(0,tf,1000);
X = [                   (xdot0*sin(om*t))/om - x0*(3*cos(om*t) - 4) - (2*ydot0*(cos(om*t) - 1))/om;
 y0 + x0*(6*sin(om*t) - 6*om*t) + (ydot0*(4*sin(om*t) - 3*om*t))/om + (2*xdot0*(cos(om*t) - 1))/om;
                                                               z0*cos(om*t) + (zdot0*sin(om*t))/om;
                                           xdot0*cos(om*t) + 2*ydot0*sin(om*t) + 3*om*x0*sin(om*t);
                          ydot0*(4*cos(om*t) - 3) - x0*(6*om - 6*om*cos(om*t)) - 2*xdot0*sin(om*t);
                                                                 zdot0*cos(om*t) - om*z0*sin(om*t)];
                                                             
figure(1)
plot3(X(1,:), X(2,:), X(3,:), 'k')
grid on
hold on
plot3(X(1,:), X(2,:), zeros(size(t)), 'r')
plot3(X(1,1), X(2,1), X(3,1), 'k*', 'LineWidth', 3)
plot3(X(1,end), X(2,end), X(3,end), 'g*', 'LineWidth', 3)
xlabel('x_{LVLH} [km]')
ylabel('y_{LVLH} [km]')
zlabel('z_{LVLH} [km]')

%integrating numerically
TSPAN = [0 tf]; %duration of integration in seconds
X0 = [x0; y0; z0; xdot0; ydot0; zdot0];
OPTIONS = odeset('Maxstep', 10);
[TOUT,YOUT] = ode45(@CWeqq,TSPAN,X0,OPTIONS);

figure(1)
plot3(YOUT(:,1), YOUT(:,2), YOUT(:,3), 'b')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

